from Acquisition import aq_parent

from Products.plonehrm.utils import apply_template_of_tool
from plonehrm.absence import AbsenceMessageFactory as _
from plonehrm.absence.utils import localize

from DateTime import DateTime


def save_note(object, event):
    """ This event is fired when an absence note is created
    or updated. If a nextContactDate has been specified in the
    note, then a new item is created in the checklist.
    """
    # Only add a checklist item if we have a date or action.
    if not object.next_contact_date and not object.next_action:
        return

    # The related absence is the parent of the note.
    absence = aq_parent(object)

    # The employee is the grandfather of the note (father is absence).
    employee = aq_parent(absence)

    # We get the employee checklist.
    content_filter = {'portal_type': 'Checklist'}
    contents = list(employee.getFolderContents(contentFilter=content_filter))

    # This shall not happen as every employee has his own checklist.
    if len(contents) == 0:
        return

    checklist = contents[0].getObject()

    # Use the next action text if specified.
    # We can be sure that this is set due to the if statement at the
    # top *and* the fact that the exit form requires a nextAction if
    # the nextContactDate is entered.
    title = object.next_action

    # We create a link to add a new note in the absence.
    link_url = absence.absolute_url() + '/createObject?type_name=Note'
    link_title = _('msg_new_note_in_absence',
                   default=u'Create a new note in the related absence')
    link_title = localize(object, link_title)

    # If a next contact date is set, use it for the note.
    date = None
    if object.next_contact_date:
        # We cast python datetime to zope DateTime.
        d1 = object.next_contact_date
        date = DateTime(d1.year, d1.month, d1.day, d1.hour, d1.minute)

    # First, we check if there exists a previous item in the checklist
    # which tells to call the employee back.
    items = checklist.findItems(title, startswith = True)

    # If such an item exists, we delete it.
    for item in items:
        checklist.deleteItem(item)

    # We add the new item to the checklist.
    checklist.addItem(title,
                      date=date,
                      link_url = link_url,
                      link_title = link_title)


def apply_template(object, event):
    """After initializing an evaluation interview, set text based
    on template.
    """
    apply_template_of_tool(object, 'portal_absence')



from DateTime import DateTime
from datetime import date
from kss.core import kssaction
from plone.app.kss.plonekssview import PloneKSSView
from Acquisition import Explicit
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import getSiteEncoding
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.interface import Interface
from zope.component import getMultiAdapter
from zope.i18n import translate

from plonehrm.absence.interfaces import IAbsenceAdapter
from plonehrm.absence import AbsenceMessageFactory as _
from plonehrm.absence.absence import AbsenceDateError

MAXITEMS = 3


class IAbsenceListViewlet(Interface):

    def get_absence_ist():
        """Return a list of (maximum is MAXITEMS) absences
        """

    def show_absence_form():
        """Return true if the absence form should be shown
        """

    def show_absence_list():
        """Return true if the absence list should be shown
        """

    def get_current_absence():
        """Return the current absence, if any.
        """


class IAddAbsenceView(Interface):
    """Marker interface to add absencelist
    """


def simple_add_absence(context, text, start_date=None):
    if text is None or text =='':
        message = _(u'msg_no_input', default=u'No input provided')
    else:
        absencelist = IAbsenceAdapter(context)
        if not isinstance(text, unicode):
            text = unicode(text, getSiteEncoding(context))
        if start_date is None:
            start_date = date.today()
        try:
            absencelist.add_absence(text, start_date)
            message = None
        except AbsenceDateError, exc:
            message = exc
    return message


def simple_close_absence(context, length=None, end_date = None):
    """ close the open absence
    """
    absencelist = IAbsenceAdapter(context)
    if end_date is None:
        end_date = date.today()
    try:
        absencelist.close_absence(end_date, length)
        message = None
    except AbsenceDateError, exc:
        message = exc
    return message


class AddAbsence(BrowserView):
    """Add an Absence.

    Note that this is the non-kss version.  We may want to concentrate
    on the kss version instead (see below).
    """

    def __call__(self):
        """Create a new Absence if the inline form is used.

        Only react if the form was submitted
        Only react if someone filled in text for the Absence
        Make a new Absence with the correct text and date.
        When not set, the date will be set to today.
        """
        text = self.request.get('text', None)
        start_date = self.request.get('new_absence_date', None)
        if start_date:
            year = int(self.request.get('new_absence_date_year'))
            month = int(self.request.get('new_absence_date_month'))
            day = int(self.request.get('new_absence_date_day'))
            start_date = date(year, month, day)
        else:
            start_date = date.today()
        message = simple_add_absence(self.context, text, start_date)
        self.context.plone_utils.addPortalMessage(message)
        response = self.request.response
        here_url = self.context.absolute_url()
        response.redirect(here_url)


class AbsenceViewlet(Explicit):
    """A viewlet for seeing and adding absencelist.
    """

    template = ViewPageTemplateFile('absence_viewlet.pt')

    def __init__(self, context, request, view=None, manager=None):
        self.context = context
        self.request = request
        self.view = view
        self.form_state = False
        self.close_form_state = False
        self.edit_form_state = False
        self.error_message = None

        # List of possible status, each one defines how the viewlet
        # is rendered.
        self.status_list = ['list',
                            'add_form',
                            'edit_form',
                            'close_form']
        self.status = 'list'

        self.manager = manager
        membership = getToolByName(self.context, 'portal_membership')
        self.checkPermission = membership.checkPermission
        self.encoding = getSiteEncoding(context)

    def set_status(self, status):
        """ Changes the way the viewlet is rendered.
        """
        if not self.can_edit_absences():
            self.status = 'list'
        elif status in self.status_list:
            self.status = status

    def get_absence_list(self):
        absences = IAbsenceAdapter(self.context)
        absencelist = absences.absencelist
        result = absencelist[-MAXITEMS:]
        result.reverse()
        return result

    def get_current_absence(self):
        absences = IAbsenceAdapter(self.context)
        return absences.current_absence()

    def show_absence_list(self):
        return self.status == 'list'

    def can_edit_absences(self):
        return self.checkPermission('plonehrm: Add absence', self.context)

    def allow_adding_absence(self):
        if not self.can_edit_absences():
            return False
        if IAbsenceAdapter(self.context).is_absent():
            return False
        return True

    def show_close_absence_button(self):
        return IAbsenceAdapter(self.context).is_absent()

    def show_add_absence_button(self):
        if not self.allow_adding_absence():
            return False
        return self.status == 'list'

    def show_absence_form(self):
        if not self.allow_adding_absence():
            return False
        return self.status == 'add_form'

    def show_absence_close_form(self):
        if not self.can_edit_absences():
            return False       
        return self.status == 'close_form'

    def show_absence_edit_form(self):
        if not self.can_edit_absences():
            return False       
        return self.status == 'edit_form'

    def render(self):
        val = self.template()
        #if not isinstance(val, unicode):
        #    val = unicode(val, self.encoding)
        return val


class AbsenceList(BrowserView):
    """ A view for listing the complete absencelist of an employee."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_absence_list(self):
        absences = IAbsenceAdapter(self.context)
        absencelist = absences.absencelist
        # Create a copy and reverse that copy, instead of mistakenly
        # reversing the original.
        result = absencelist[:]
        result.reverse()
        return result


class AbsenceKss(PloneKSSView):
    """ This class contains every KSS views used in the absence
    viewlet.
    """

    @property
    def lang(self):
        props = getToolByName(self.context, 'portal_properties')
        return props.site_properties.getProperty('default_language')

    def refresh_viewlet(self, status = 'list'):
        """ Sets the view to the asked status and refreshes it.
        """
        core = self.getCommandSet('core')
        selector = core.getHtmlIdSelector('absencelist')

        # Get and configure the view.
        view = self.context.restrictedTraverse('@@plonehrm.absence')
        view.set_status(status)

        # Display it.
        rendered = view.render()
        core.replaceHTML(selector, rendered)

    @kssaction
    def show_list(self):
        """ Shows the list of absences.
        """
        self.refresh_viewlet('list')

    @kssaction
    def show_add_form(self):
        """ Shows the form to add an absence.
        """
        self.refresh_viewlet('add_form')

    @kssaction
    def show_edit_form(self):
        """ Shows the form to edit an absence.
        """
        self.refresh_viewlet('edit_form')

    @kssaction
    def show_close_form(self):
        """ Shows the form to close an absence.
        """
        self.refresh_viewlet('close_form')

    @kssaction
    def cancel(self):
        """ Hides the existing status message and refreshes
        the viewlet.
        """
        self.getCommandSet('plone').issuePortalMessage('')
        self.show_list()


    def validate_add_or_edit_form(self, form):
        """ Checks that the form submitted when adding
        or editing an absence is correct.
        """

        # Checks that every field has been submitted.
        fields = ['text',
                  'new_absence_date',
                  'new_absence_date_year',
                  'new_absence_date_month',
                  'new_absence_date_day']
        for field in fields:
            if not field in form:
                # Should not happen.
                return
            
        errors = []
        # Check that a title has been provided.
        if not form['text']:
            errors.append('absence_viewlet_error_no_description')

        # Check that the date is valid.
        if form['new_absence_date']:
            year = int(form['new_absence_date_year'])
            month = int(form['new_absence_date_month'])
            day = int(form['new_absence_date_day'])

            try:
                start_date = date(year, month, day)
            except:
                errors.append('absence_viewlet_error_invalid_date')
                start_date = date.today()
        else:
            start_date = date.today()

        # Check that the start date is tomorrow or earlier.
        if start_date.toordinal() > date.today().toordinal() + 1:
            errors.append('absence_viewlet_error_date_future')

        return (start_date, errors)

    def clear_errors(self):
        """ Hides every errors displayed in the form.
        """
        errors = ['absence_viewlet_error_no_description',
                  'absence_viewlet_error_invalid_date',
                  'absence_viewlet_error_conflict_date',
                  'absence_viewlet_error_date_future',
                  'absence_viewlet_error_lenght_null',
                  'absence_viewlet_error_lenght_no_integer',
                  'absence_viewlet_error_end_date_before_start_date']

        core = self.getCommandSet('core')
        for error in errors:
            selector = core.getHtmlIdSelector(error)
            core.setAttribute(selector, 'class', 'dont-show')

        selector = core.getHtmlIdSelector('absence_viewlet_global_error')
        core.setAttribute(selector, 'class', 'dont-show')
        core.replaceInnerHTML(selector, '')
        

    def show_errors(self, errors = [], global_error = ''):
        """ Show errors found while validating the form.
        """
        # Clears the previous errors.
        self.clear_errors()
        
        core = self.getCommandSet('core')
        for error in errors:
            selector = core.getHtmlIdSelector(error)
            core.setAttribute(selector, 'class', 'errormessage')

        if global_error:
            selector = core.getHtmlIdSelector('absence_viewlet_global_error')
            core.setAttribute(selector, 'class', 'errormessage')
            core.replaceInnerHTML(selector, global_error)

        status = translate(_(u'msg_absence_error',
                             u'Error have been found while saving the absence'),
                           target_language = self.lang)
        
        self.getCommandSet('plone').issuePortalMessage(status,
                                                       msgtype='Error')

    @kssaction
    def add_absence(self):
        """ Creates the new absence and displays the list again.
        """
        form = self.request.form

        try:
            start_date, errors = self.validate_add_or_edit_form(form)
        except TypeError:
            # Happens when validate_form returned None, meaning
            # that the form has not been correctly submitted.
            # This should not happen.
            return

        # Checks that the date does not conflict with the previous
        # absence.
        absence_list_view = self.context.restrictedTraverse('@@absencelist')
        absence_list = absence_list_view.get_absence_list()

        if (absence_list and
            start_date.toordinal() <= \
            absence_list[0].end_date.toordinal()):
            errors.append("absence_viewlet_error_conflict_date")

        if errors:
            self.show_errors(errors = errors)
            return

        # Now we can add the absence.
        message = simple_add_absence(self.context, form['text'], start_date)

        if message:
            # Should not happen.
            self.show_errors(global_error = message)
            return

        # Display the status success message.
        status = translate(_(u'msg_absence_added',
                             u'Absence added'),
                           target_language = self.lang)
        self.getCommandSet('plone').issuePortalMessage(status)

        # Shows the list again.
        self.show_list()

    @kssaction
    def edit_absence(self):
        """ Updates the current absence text and start date.
        """
        form = self.request.form
        try:
            start_date, errors = self.validate_add_or_edit_form(form)
        except TypeError:
            # Happens when validate_form returned None, meaning
            # that the form has not been correctly submitted.
            # This should not happen.
            return

        # Get the absence list.
        absence_list_view = self.context.restrictedTraverse('@@absencelist')
        absence_list = absence_list_view.get_absence_list()
        if not absence_list:
            # Should not happen.
            return
        
        # Checks that the date does not conflict with the previous
        # absence. As we edit the current absence (which is the last one),
        # the previous absence is absencelist[1].
        if (len(absence_list) > 1 and
            start_date.toordinal() <= \
            absence_list[1].end_date.toordinal()):
            errors.append("absence_viewlet_error_conflict_date")

        if errors:
            self.show_errors(errors = errors)
            return

        current_absence = absence_list[0]
        current_absence.title = form['text']
        current_absence.start_date = start_date

        # Display the status success message.
        status = translate(_(u'msg_absence_edited',
                             u'Absence edited'),
                           target_language = self.lang)
        self.getCommandSet('plone').issuePortalMessage(status)

        # Shows the list again.
        self.show_list()

    @kssaction
    def close_absence(self):
        """ Closes the current absence.
        """
        form = self.request.form

        absence_list_view = self.context.restrictedTraverse('@@absencelist')
        absence_list = absence_list_view.get_absence_list()

        if not absence_list:
            # Should not happen.
            return
        current_absence = absence_list[0]

        # Checks that every fields 
        fields = ['absence_length',
                  'close_absence_date',
                  'close_absence_date_year',
                  'close_absence_date_month',
                  'close_absence_date_day']

        for field in fields:
            if not field in form:
                # Should not happen.
                return

        errors = []

        # Check that the length is valid.
        try:
            absence_lenght = int(form['absence_length'])
        except:
            errors.append('absence_viewlet_error_lenght_no_integer')
            absence_lenght = 1

        if absence_lenght < 1:
            errors.append('absence_viewlet_error_lenght_null')

        # Check that the date is valid and correct.
        if form['close_absence_date']:
            year = int(form['close_absence_date_year'])
            month = int(form['close_absence_date_month'])
            day = int(form['close_absence_date_day'])

            try:
                end_date = date(year, month, day)
            except:
                errors.append('absence_viewlet_error_invalid_date')
                end_date = date.today()
        else:
            end_date = date.today()
        
        # Check that the end date is after the start date.
        if end_date.toordinal() < current_absence.start_date.toordinal():
            errors.append('absence_viewlet_error_end_date_before_start_date')
            
        if errors:
            self.show_errors(errors=errors)
            return

        # Close the absence.
        message = simple_close_absence(self.context, form['absence_length'],
                                       end_date)

        if message:
            # Should not happen.
            self.show_errors(global_error = message)
            return

        # Display the status success message.
        status = translate(_(u'msg_absence_closed',
                             u'Absence closed'),
                           target_language = self.lang)
        self.getCommandSet('plone').issuePortalMessage(status)

        # Shows the list again.
        self.show_list()



__author__ = """Jean-Paul Ladage <j.ladage@zestsoftware.nl>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Acquisition import aq_inner, aq_parent
from Products.Archetypes import atapi
from Products.CMFCore.utils import ImmutableId
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import ModifyPortalContent

import config

absenceevaluation_schema = atapi.BaseFolderSchema.copy()


class AbsenceTool(ImmutableId, atapi.BaseFolder):
    """ Absence Tool, stores EvaluationInterview templates.
    """
    security = ClassSecurityInfo()
    __implements__ = (atapi.BaseFolder.__implements__, )

    id = 'portal_absence'
    typeDescription = "Absence evaluations templates"
    typeDescMsgId = 'description_edit_absencetool'
    schema = absenceevaluation_schema

    def __init__(self, *args, **kwargs):
        self.setTitle('Absence Evaluations Templates')

    security.declareProtected(ModifyPortalContent, 'indexObject')
    def indexObject(self):
        pass

    security.declareProtected(ModifyPortalContent, 'reindexObject')
    def reindexObject(self, idxs=[]):
        pass

    security.declareProtected(ModifyPortalContent, 'reindexObjectSecurity')
    def reindexObjectSecurity(self, skip_self=False):
        pass

    def listTemplates(self, type='absence_evaluation'):
        """List the templates in this tool.

        Optionally list the templates from a higher level
        portal_contracts tool as well.  This is done by trying to call
        getUseHigherLevelTemplates on the current tool.  This method
        is not available by default, but can be implemented by third
        party products.  Default is False.  If you always want this to
        be True, add a python script getUseHigherLevelTool in the
        portal_skins/custom folder that just returns True.

        We return full objects; returning just ids will fail as we
        then have no way of knowing in which portal_contracts that
        template id is.
        """
        templates = self.contentValues()

        try:
            recursive = self.getUseHigherLevelTool()
        except AttributeError:
            recursive = False

        if recursive:
            # Get our grand parent and ask him for a portal_contracts
            # tool.  Note that if we ask our parent, then a
            # getToolByName will return ourselves, which is not what
            # we want.
            grand_parent = aq_parent(aq_parent(aq_inner(self)))
            higher_tool = getToolByName(grand_parent, self.id, None)
            if higher_tool is not None:
                templates += higher_tool.listTemplates()

        # We now filter the template to keep only Template object
        # for which the type is the one asked.
        filtered = []

        for template in templates:
            try:
                if template.getType() == type:
                    filtered.append(template)
            except:
                # Old templates are left in the folder.
                pass

        return filtered


atapi.registerType(AbsenceTool, config.PROJECTNAME)



"""Definition of the Absence content type
"""

from datetime import date
from Acquisition import aq_chain
from zope.interface import implements
from Products.Archetypes import atapi
from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content import schemata

from plonehrm.absence import AbsenceMessageFactory as _
from plonehrm.absence.interfaces import IAbsence
from Products.plonehrm.interfaces import IEmployee
from plonehrm.absence.config import PROJECTNAME
from zope import component

AbsenceSchema = folder.ATFolderSchema.copy() + atapi.Schema((

    atapi.DateTimeField(
        name='startDate',
        widget=atapi.CalendarWidget(
            label=_(u'label_start_date', default=u'Start date'),
            show_hm = False,
        )
    ),

    atapi.DateTimeField(
        name='endDate',
        widget=atapi.CalendarWidget(
            label=_(u'label_end_date', default=u'End date'),
            show_hm = False,
        )
    ),

    atapi.IntegerField(
        name='absence_length',
        widget=atapi.IntegerWidget(
            label=_(u'label_days_absent', default=u'Absence length (days)'),
        )
    ),

))

# Set storage on fields copied from ATFolderSchema, making sure
# they work well with the python bridge properties.

AbsenceSchema['title'].storage = atapi.AnnotationStorage()
AbsenceSchema['description'].storage = atapi.AnnotationStorage()
AbsenceSchema['description'].widget.visible = {'view': 'visible',
                                               'edit': 'invisible'}
schemata.finalizeATCTSchema(
    AbsenceSchema,
    folderish=True,
    moveDiscussion=False,
    )


class Absence(folder.ATFolder):
    """Absence"""
    implements(IAbsence)

    meta_type = "Absence"
    schema = AbsenceSchema

    title = atapi.ATFieldProperty('title')
    description = atapi.ATFieldProperty('description')
    # XXX Make 'text' a rich text field?
    text = atapi.ATFieldProperty('title')
    start_date = atapi.ATDateTimeFieldProperty('startDate')
    end_date = atapi.ATDateTimeFieldProperty('endDate')

    def __repr__(self):
        value = "<Absence: %s" % self.text
        if self.start_date:
            # We get the (python) date instead of the datetime here.
            value += ", " + self.start_date.date().isoformat()
        if self.end_date:
            value += " - " + self.end_date.date().isoformat()
        return value + ">"

    def get_employee(self):
        """Get the employee that this contract is in.

        Note that we probably are in the portal_factory, so our direct
        parent is not an Employee.  But we can traverse up the
        acquisition chain to find it.
        """
        for parent in aq_chain(self):
            if IEmployee.providedBy(parent):
                return parent

    def days_absent(self):
        """How long did the absence take?

        If the absence has ended, return the length stored on the
        object. Else try to estimate the number of days someone hasn't
        worked.

        Note 1: if the start date of the absence is in the future,
        return 1.

        Note 2: due to the way the algorithm works, should the
        employee works a 0 hour/day contract the employee will get 0
        sick days by his/her name when they are sick or were sick.
        """

        if self.end_date:
            return self.get('absence_length', 0)

        end_date = date.today()
        if end_date.toordinal() < self.start_date.toordinal():
            # Apparently the absence starts in the future. Since
            # (currently) absences can only be started one day in the
            # future, we'll count this case as the employee being sick
            # a single day.  the employee is absent for a day.
            return 1

        parent = self.get_employee()
        if parent is None:
            return
        view = component.getMultiAdapter((parent, self.REQUEST),
                                         name=u'contracts')
        current = view.current_contract()
        days_per_week = 0
        if current is not None:
            days_per_week = current.getDaysPerWeek()
        if days_per_week == 0:
            days_per_week = 5

        absence_length = float(end_date.toordinal() -
                               self.start_date.toordinal() + 1)
        sickness_days = (days_per_week / 7.0) * absence_length
        rounded_sickness = int(round(sickness_days))
        return rounded_sickness

    @property
    def is_current_absence(self):
        """ Returns True is the absence is not closed and
        False in the other cases.
        """
        return not bool(self.end_date)

atapi.registerType(Absence, PROJECTNAME)



import unittest
from Testing import ZopeTestCase as ztc
from zope.testing import doctest
from plonehrm.absence.tests.base import BaseTestCase


def test_suite():
    return unittest.TestSuite([
        # Integration tests that use PloneTestCase
        ztc.ZopeDocFileSuite(
                'docs/absence.txt', package='plonehrm.absence',
                test_class=BaseTestCase),
        ztc.ZopeDocFileSuite(
                'docs/checker.txt', package='plonehrm.absence',
                test_class=BaseTestCase),
        ])


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



from zope.i18n import translate
from Products.CMFCore.utils import getToolByName


def localize(context, text):
    props = getToolByName(context, 'portal_properties')
    lang = props.site_properties.getProperty('default_language')
    return translate(text, target_language=lang)





from setuptools import setup, find_packages
import os

version = '1.1'

history_path = os.path.join('plonehrm', 'absence', 'HISTORY.txt')
history_file = open(history_path)
history = history_file.read().strip()
history_file.close()



setup(name='plonehrm.absence',
      version=version,
      description="Register employee absence",
      long_description=history,
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='',
      author='Zest Software',
      author_email='info@zestsoftware.nl',
      url="",
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['plonehrm'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'Products.plonehrm>=2.7',
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )
